%% Run UCI experments
%% setup parallel computing
%matlabpool
parpool

%% addpath for solver routines
addpath(genpath('../gtf_code'))


%% preparations
% get list of names for data
fileID = fopen('datasetNames1.txt');
datanames=textscan(fileID,'%s');
fclose(fileID);
datanames=datanames{1};

numdata=length(datanames);
rep=10;% 10 repeats
K=5; % KNN
numseeds=5; % num of seeds for each class. if less than 5, take all.

% grid of paramters.
lambda_list=-7:0.5:-2;
% not large enough, not fine enough?
lambda_list=exp(lambda_list); 

lambda_list2=-10:1:0;
lambda_list2=exp(lambda_list2); 

sz=length(lambda_list);

% declare results holder
klist=[0,1,2];
numk=length(klist);
% for GTF completion.
mcr_results=zeros(sz,rep,numk,numdata); % misclassification rate
mrr_results=zeros(sz,rep,numk,numdata);% mean reciprocal rank
dof_results=zeros(sz,rep,numk,numdata);

% for MAP.
mcr_results2=zeros(sz,rep,numdata);
mrr_results2=zeros(sz,rep,numdata);
dof_results2=zeros(sz,rep,numdata);







%% for each data

rng(15213);
for di = 1:numdata
    load([datanames{di},'/data.mat']);% got label Y
    load([datanames{di},'/graph.mat']);% got edges1, edges2 and w
    % w is all 1 so it is an unweighted graph.
    true_labels=data(:,end);
    C=max(true_labels);
    n=length(true_labels);
    m=length(edges1);
    
    
    mu=0.01;
    R=ones(n,1)/C;% +0.01*randn(n,1); % the additional noise is there to ensure gtf
    % can have high dof. Basically something that makes the plots nicer.
    
    % Need this to calculate d.o.f for the laplacian smoothing
    D = sparse((1:m)',edges1,1,m,n,10*m);
    D = D+ sparse((1:m)',edges2,-1,m,n);

    
    % Assume the graph has a single connected component.
    
    fprintf('Start experimenting on dataset:[%d] %s .\n',di,datanames{di});
    
    % load data, run CV to get parameters.
    for r=1:rep
         fprintf('--------------Rep %d-----------\n',r);
        % Generate seeds
        mask=false(n,1);
        Y=zeros(n,C);
        
        fprintf('--------------Generating seeds----------- \n');
        for i=1:C
            idx=find(true_labels==i);
            if length(idx)>numseeds
                idx_s=randsample(length(idx),numseeds);
                idx_s=idx(idx_s);
            else
                idx_s=idx;
            end
            mask(idx_s)=1;
            Y(idx_s,i)=1;
        end
        Y(~mask,:)=-1;
        
                % Other methods
        parfor i=1:sz % solve to all lambda in the list
            
             fprintf('--------------Run MAD ----Dataset %d, Rep %d, lambda %d \n',di,r,i);

            lambda=lambda_list2(i)*100; 
            % we are multiplying 100 to approximately match the dof
            
%            Yhat=ones(n,C);
            keep_trying = true; try_count=0;

            while keep_trying
                    keep_trying = false;

                    % file_name = ask user for file name

                     try
                         [ Yhat ] = MAD( Y, [edges1,edges2], w , lambda,mu,R );
            
                     catch
                          if try_count<10
                              keep_trying=true;
                              fprintf('Failed, but try again.\n')
                              try_count=try_count+1;

                          else
                              keep_trying=false;
                              fprintf('Failed for 10 times... giving up.\n')
                          end
                     end

            end

            mcr_results2(i,r,di)=calculateMCR( true_labels, Yhat,mask);
            mrr_results2(i,r,di)=calculateMRR( true_labels, Yhat,mask);
            
            %solve for DOF
            diagLambda=diag(lambda*(D'*D))+mask+mu;
%            diagLambda=diag(Lambda);

            %Approximating DOF given the fact that the matrix id SDD.
            dof = sum(1./diagLambda);
            dof_results2(i,r,di)=dof;
        end
        
        % GTF
        parfor ki=1:numk

            k=klist(ki);
            for i=1:sz % solve to all lambda in the list
                            fprintf('--------------Run GTF k=%d----Dataset %d, Rep %d,lambda %d \n',ki,di,r,i);
                lambda=lambda_list2(i);
                CVX=0;% solve it with high numerical accuracy using CVX
%                Yhat=ones(n,C);
%                dof=1;

              [ Yhat, dof ] = MAD_gtf( Y, [edges1,edges2], w, lambda,k,mu,R,CVX);

                mcr_results(i,r,ki,di)=calculateMCR( true_labels, Yhat,mask);
                mrr_results(i,r,ki,di)=calculateMRR( true_labels, Yhat,mask);

                dof_results(i,r,ki,di)=dof;
            end
        end
    save('UCI_results.mat','lambda_list','dof_results','dof_results2','mcr_results','mcr_results2','mrr_results','mrr_results2');

    end
end
%% Save results

    save('UCI_results.mat','lambda_list','dof_results','dof_results2','mcr_results','mcr_results2','mrr_results','mrr_results2');
